@@ -10,6 +10,7 @@ import os
 from typing import Dict
 from typing import List
 from typing import Optional
+from typing import Set
 
 import hyperthought as ht
 from hyperthought.metadata import MetadataItem
@@ -328,6 +329,7 @@ class Manifest:
 
         Ancestor folders will be created as needed.
         """
+        # TODO:  Consider removing this function and using add_paths instead.
         path = utils.clean_path(path)
         self._validate_path(path)
 
@@ -430,6 +432,78 @@ class Manifest:
         # Create all records.
         self.database.bulk_load_files(file_data=file_data)
 
+    def add_paths(self, paths: List[str]) -> None:
+        """
+        Add paths to the manifest.
+
+        Ancestor folders will be created as needed.
+        """
+        # TODO:  Make paths type generic. ^^^
+        paths = {utils.clean_path(path) for path in paths}
+
+        for path in paths:
+            self._validate_path(path)
+
+        len_ignorepath = (
+            0
+            if self._ignore_path is None
+            else len(self._ignore_path)
+        )
+
+        def get_ancestor_paths(path):
+            ancestor_tokens = (
+                path[len_ignorepath:].strip(os.path.sep).split(os.path.sep)
+            )[:-1]
+
+            for index in range(1, len(ancestor_tokens) + 1):
+                yield os.path.join(
+                    self._ignore_path,
+                    *ancestor_tokens[:index]
+                )
+
+        for path in paths:
+            for ancestor_path in get_ancestor_paths(path):
+                paths.add(ancestor_path)
+
+        existing_paths = set()
+        path_to_hyperthought_id = {}
+
+        for file_ in self.database.get_files(paths=paths):
+            existing_paths.add(file_["path"])
+            path_to_hyperthought_id[file_["path"]] = file_["hyperthought_id"]
+
+        for path in paths:
+            if path not in existing_paths:
+                path_to_hyperthought_id[path] = utils.generate_id()
+
+        def get_hyperthought_id_path(path):
+            hyperthought_id_path = ","
+
+            for ancestor_path in get_ancestor_paths(path):
+                ancestor_hyperthought_id = (
+                    path_to_hyperthought_id[ancestor_path])
+                hyperthought_id_path += f"{ancestor_hyperthought_id},"
+
+            return hyperthought_id_path
+
+        paths_to_create = paths - existing_paths
+        file_data = []
+
+        for path in paths_to_create:
+            file_data.append({
+                "name": os.path.basename(path),
+                "hyperthought_id": path_to_hyperthought_id[path],
+                "hyperthought_id_path": get_hyperthought_id_path(path),
+                "is_folder": os.path.isdir(path),
+                "path": path,
+                "end_bytes": None,
+                "size": None,
+                "file_hash": None,
+            })
+
+        # Create all records.
+        self.database.bulk_load_files(file_data=file_data)
+
     def remove_path(self, path: str) -> None:
         """
         Remove a path from the manifest.
@@ -473,6 +547,7 @@ class Manifest:
 
     def has_path(self, path: str) -> bool:
         """Determine whether a path has already been added."""
+        # TODO:  Consider removing this function and always using has_paths.
         path = utils.clean_path(path)
         file_id = self.database.get_file_id(path=path)
         return file_id is not None
@@ -575,7 +650,7 @@ class Manifest:
             metadata_application_data=metadata_application_data
         )
 
-    def to_json(self, compute_file_info: bool = True) -> dict:
+    def to_json(self, compute_file_info: bool = True) -> Dict:
         """Convert manifest database to JSON and return results."""
         if compute_file_info and not self.file_info_computed:
             self.compute_file_info()
@@ -773,7 +848,7 @@ def from_json(manifest_data: Dict, manifest_database_file: str) -> Manifest:
         manifest_data["files"],
         key=lambda item: item["path"],
     )
-    len_ignore_path = len(manifest.ignore_path)
+    len_ignore_path = len(manifest.ignore_path) if manifest.ignore_path else 0
     path_to_hyperthought_id = {}
     manifest_id_to_hyperthought_id = {}
     file_data = []
