@@ -6,8 +6,10 @@ from enum import IntEnum
 import os
 import sqlite3
 from typing import Dict
+from typing import Generator
 from typing import List
 from typing import Optional
+from typing import Set
 
 import hyperthought as ht
 from hyperthought.metadata import MetadataItem
@@ -516,6 +518,7 @@ class Database:
         -------
         A list of dicts containing information on all folders in the job.
         """
+        # TODO:  Use generator.
         cursor = self.connection.cursor()
         sql = "SELECT * FROM File WHERE is_folder"
         cursor.execute(sql)
@@ -535,6 +538,7 @@ class Database:
         -------
         A list of dicts containing information on all files in the job.
         """
+        # TODO:  Use generator.
         cursor = self.connection.cursor()
         sql = "SELECT * FROM File WHERE NOT is_folder"
         cursor.execute(sql)
@@ -547,7 +551,7 @@ class Database:
         ]
 
     def get_file_id(self, path: str) -> Optional[int]:
-        """Determine whether a given file exists in the database."""
+        """Get an id for a File record given a path."""
         cursor = self.connection.cursor()
         sql = """
         SELECT id
@@ -648,27 +652,57 @@ class Database:
 
         return {key: row[key] for key in row.keys()}
 
-    def get_files(self, file_ids: List[int]) -> List[Dict]:
-        """Get all file records corresponding a list of ids."""
+    def get_files(
+        self,
+        file_ids: Optional[List[int]] = None,
+        paths: Optional[List[str]] = None
+    ) -> Generator[Dict, Optional[None], Optional[None]]:
+        """
+        Get all file records corresponding a list of ids or paths.
+
+        Parameters
+        ----------
+        file_ids : list of int
+            Database ids for the files of interest.
+        paths : list of str
+            Paths for files of interest.
+            Must be provided if file_ids is not.
+
+        Returns
+        -------
+        A list of dicts containing file record data.
+        """
+        if not file_ids and not paths:
+            raise ValueError("file_ids or paths must be provided")
+
+        if file_ids:
+            sql = "SELECT * FROM File WHERE id IN ("
+            items = file_ids
+        else:
+            sql = "SELECT * FROM File WHERE path IN ("
+            items = paths
+
+        params = {}
+        key_index = 0
+
+        for item in items:
+            key = f"item_{key_index}"
+            sql += f":{key}, "
+            params[key] = item
+
+        sql = sql[:-2] + ")"  # Remove trailing ", " and close parenthesis.
         cursor = self.connection.cursor()
-        sql = "SELECT * FROM File WHERE id IN ("
+        cursor.execute(sql, params)
 
-        for file_id in file_ids:
-            sql += f"{file_id}, "
-
-        # Remove trailing ", " and close parenthesis.
-        sql = sql[:-2] + ")"
-
-        cursor.execute(sql)
-        return [
-            {
+        for row in cursor.fetchall():
+            yield {
                 key: row[key]
                 for key in row.keys()
             }
-            for row in cursor.fetchall()
-        ]
 
-    def get_all_metadata_application(self) -> List[Dict]:
+        cursor.close()
+
+    def get_all_metadata_application(self) -> Generator[Dict, Optional[None], Optional[None]]:  # noqa: E501
         """Get all MetadataApplication records."""
         cursor = self.connection.cursor()
         sql = """
@@ -687,7 +721,7 @@ class Database:
                 for key in row.keys()
             }
 
-    def get_all_parsers(self) -> List[Dict]:
+    def get_all_parsers(self) -> Generator[Dict, Optional[None], Optional[None]]:  # noqa: 501
         """Get all Parser records."""
         cursor = self.connection.cursor()
         sql = """
@@ -706,7 +740,7 @@ class Database:
                 for key in row.keys()
             }
 
-    def get_all_metadata(self, file_specific: bool = True) -> List[Dict]:
+    def get_all_metadata(self, file_specific: bool = True) -> Generator[Dict, Optional[None], Optional[None]]:  # noqa: 501
         """
         Get all Metadata records.
 
@@ -798,7 +832,8 @@ class Database:
         """
         self._update(table="File", id_=id_, updates=updates, commit=commit)
 
-    def reset_file_statuses(self,
+    def reset_file_statuses(
+        self,
         existing_statuses: List[FileStatus],
         new_status: FileStatus,
         commit: bool = True,
@@ -1247,7 +1282,8 @@ class Database:
         result = cursor.fetchone()
         return result["file_count"]
 
-    def update_folder_hyperthought_id(self,
+    def update_folder_hyperthought_id(
+        self,
         old_id: str,
         new_id: str,
         commit: bool = True,
